/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: MonitoringDummy.java 702 2011-12-08 17:28:34Z dangtran $
 *
 */

package org.ow2.sirocco.vmm.agent.monitoring.driver.dummy;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;
import org.ow2.sirocco.vmm.agent.domain.AbstractHost;
import org.ow2.sirocco.vmm.agent.domain.AbstractVirtualMachine;
import org.ow2.sirocco.vmm.agent.domain.ManagedResource;
import org.ow2.sirocco.vmm.agent.domain.ServerPool;
import org.ow2.sirocco.vmm.agent.monitoring.api.MonitoringService;
import org.ow2.sirocco.vmm.api.VMMException;
import org.ow2.sirocco.vmm.api.monitoring.PerfMetric;
import org.ow2.sirocco.vmm.api.monitoring.PerfMetricInfo;
import org.ow2.sirocco.vmm.api.monitoring.PerfMetricInfo.Unit;

public class MonitoringDummy implements MonitoringService {
    static Logger logger = Logger.getLogger(MonitoringDummy.class);

    private Map<String, String> attributes;

    /**
     * Constructor
     * 
     * @param attributes
     * @param objectname
     * @param name
     * @param MBeanServerName
     * @throws Exception
     */
    public MonitoringDummy(final Map<String, String> attributes) {
        this.attributes = new HashMap<String, String>(attributes);
    }

    @Override
    public Map<String, String> getAttributes() {
        return this.attributes;
    }

    /**
     * Start and initialize the monitoring service
     * 
     * @param args : arguments to start the monitoring service
     */
    public void start() {
        MonitoringDummy.logger.info("Monitoring Driver dummy start");
    }

    /**
     * Stop the monitoring service
     * 
     * @param args
     */
    public void stop() {
        MonitoringDummy.logger.info("Monitoring Driver dummy stopped");
    }

    /**
     * Retrieve the list of metrics of this host
     * 
     * @param host : the name of the host
     * @return String[] : list of metrics of this host if there is or null
     */
    public PerfMetricInfo[] listPerfMetrics(final ManagedResource resource) {
        try {
            ArrayList<PerfMetricInfoDummy> infos = new ArrayList<PerfMetricInfoDummy>();
            int cptcpu = 0;

            if (resource instanceof AbstractHost || resource instanceof ServerPool) {
                AbstractHost host = (AbstractHost) resource;
                while (cptcpu < host.getNumCPU()) {
                    infos.add(new PerfMetricInfoDummy("cpu." + cptcpu + ".user", Unit.PERCENT, null, null));
                    infos.add(new PerfMetricInfoDummy("cpu." + cptcpu + ".system", Unit.PERCENT, null, null));
                    cptcpu++;
                }
            } else if (resource instanceof AbstractVirtualMachine) {
                AbstractVirtualMachine vm = (AbstractVirtualMachine) resource;
                infos.add(new PerfMetricInfoDummy("cpu.virt_cpu_total", Unit.PERCENT, vm.getStartTime(), null));
                infos.add(new PerfMetricInfoDummy("disk.hdc.disk_octets.read", Unit.BYTES_PER_SEC, null, null));
                infos.add(new PerfMetricInfoDummy("disk.hdc.disk_octets.write", Unit.BYTES_PER_SEC, null, null));
            }

            return infos.toArray(new PerfMetricInfoDummy[infos.size()]);
        } catch (VMMException e) {
            MonitoringDummy.logger.error("Error", e);
            return null;
        }
    }

    /**
     * Retrieve the value of the metadata's metric of this host
     * 
     * @param host : the name of the host
     * @param metadata : the name of the metric
     * @return String : value of this metric if there is or null
     */
    @Override
    public PerfMetric getPerfMetric(final ManagedResource resource, final String metadata) {
        PerfMetric p = new PerfMetric();
        p.setTime(new Date().getTime());
        if (metadata.contains("cpu")) {
            p.setValue(Float.valueOf(String.valueOf((Math.random() * 100))));
        } else {
            try {
                if (resource instanceof AbstractHost || resource instanceof ServerPool) {
                    AbstractHost host = (AbstractHost) resource;
                    p.setValue(Float.valueOf(String.valueOf((Math.random() * host.getTotalMemoryMB()))));
                } else if (resource instanceof AbstractVirtualMachine) {
                    AbstractVirtualMachine vm = (AbstractVirtualMachine) resource;
                    p.setValue(Float.valueOf(String.valueOf((Math.random() * vm.getMemorySizeMB()))));
                }
            } catch (VMMException e) {
                MonitoringDummy.logger.error("Resource error: " + e);
            }

        }
        return p;
    }

    /**
     * Retrieve the value of the metadata's metric of this host between the date
     * startTime and endTime with a period
     * 
     * @param host : the name of the host
     * @param metadata : the name of the metric
     * @param startTime : the date where we start retrieve values
     * @param endTime : the date where we stop retrieve values
     * @param period : the period we retrieve values
     * @return String : list of values's metric of this host if there is or null
     */
    public PerfMetric[] getPerfMetrics(final ManagedResource resource, final String name, final Date startTime,
        final Date endTime, final long period, final String type) {
        ArrayList<PerfMetric> list = new ArrayList<PerfMetric>();
        Date dateTmp = startTime;
        while (dateTmp.before(endTime)) {
            PerfMetric p = new PerfMetric();
            p.setTime(dateTmp.getTime());
            if (name.contains("cpu")) {
                p.setValue(Float.valueOf(String.valueOf((Math.random() * 100))));
            } else {

                try {
                    if (resource instanceof AbstractHost || resource instanceof ServerPool) {
                        AbstractHost host = (AbstractHost) resource;
                        p.setValue(Float.valueOf(String.valueOf((Math.random() * host.getTotalMemoryMB()))));
                    } else if (resource instanceof AbstractVirtualMachine) {
                        AbstractVirtualMachine vm = (AbstractVirtualMachine) resource;
                        p.setValue(Float.valueOf(String.valueOf((Math.random() * vm.getMemorySizeMB()))));
                    }
                } catch (VMMException e) {
                    MonitoringDummy.logger.error("Resource error: " + e);
                }

            }
            list.add(p);
            dateTmp.setTime(dateTmp.getTime() + (period * 1000L));
        }
        return list.toArray(new PerfMetric[list.size()]);
    }

    /**
     * Initialize variables when a host starts if needed
     * 
     * @param VMName
     */
    public void startHost(final String VMName) {
        MonitoringDummy.logger.info("Started  monitoring of host " + VMName);
    }

    /**
     * Initialize variables when a host stop if needed
     * 
     * @param VMName
     */
    public void stopHost(final String VMName) {
        MonitoringDummy.logger.info("Stopped  monitoring of host " + VMName);
    }

    /**
     * Initialize variables when a VM starts if needed
     * 
     * @param VMName
     */
    public void startVM(final String VMName) {
        MonitoringDummy.logger.info("Started the monitoring of VM " + VMName);
    }

    /**
     * Initialize variables when a VM stop if needed
     * 
     * @param VMName
     */
    public void stopVM(final String VMName) {
        MonitoringDummy.logger.info("Stopped the monitoring of  VM " + VMName);
    }
}
